package at.jku.ssw.queue;

import java.util.NoSuchElementException;

import at.jku.ssw.Iterator;

/**
 * A queue of objects. Queues typically order elements in a FIFO
 * (first-in-first-out) manner.
 */
public abstract class Queue {
	/**
	 * Inserts the value into this queue.
	 * 
	 * @throws IllegalStateException
	 *             if the element cannot be added at this time due to capacity
	 *             restrictions
	 */
	public abstract void enqueue(Object value);

	/**
	 * Retrieves and removes the head of this queue.
	 * 
	 * @throws NoSuchElementException
	 *             if the queue is empty
	 */
	public abstract Object dequeue();

	/**
	 * Retrieves the head of this queue without removing it.
	 * 
	 * @throws NoSuchElementException
	 *             if the queue is empty
	 */
	public abstract Object peek();

	/**
	 * Retrieves the last element of the queue, i.e., the ({@linkplain #push
	 * last} inserted element.
	 * 
	 * @throws NoSuchElementException
	 *             if the queue is empty
	 */
	public abstract Object peekLast();

	/**
	 * Returns the number of values in this queue.
	 */
	public abstract int size();

	/**
	 * Returns an iterator of the values in this queue starting with the head
	 * element ({@link #peek()}.
	 */
	public abstract Iterator iterator();
}