package at.jku.ssw.stack;

import java.util.EmptyStackException;
import java.util.NoSuchElementException;

import at.jku.ssw.Iterator;

/** A LIFO (last-in-first-out) stack of objects. */
public abstract class Stack {

	/**
	 * Pushes a value onto the top of this stack.
	 */
	public abstract void push(Object value);

	/**
	 * Removes the value at the top of this stack and returns that value as the
	 * value of this function.
	 *
	 * @throws EmptyStackException
	 *             if this stack is empty.
	 */
	public abstract Object pop();

	/**
	 * Returns the value at the top of this stack without removing it.
	 *
	 * @throws EmptyStackException
	 *             if this stack is empty.
	 */
	public abstract Object peek();

	/**
	 * Pushes the top element to the bottom of the stack.
	 *
	 * @throws NoSuchElementException
	 *             if this stack contains less than one elements.
	 */
	public abstract void pushDown();

	/**
	 * Returns the number of values on this stack.
	 */
	public abstract int size();

	/**
	 * Returns an iterator of the values on this stack, starting with the
	 * top-of-stack element ({@link #peek()}.
	 */
	public abstract Iterator iterator();

	/**
	 * Reverses the elements in this stack.
	 */
	public abstract void reverse();
}